Package: BulkSignalR
Type: Package
Title: Infer Ligand-Receptor Interactions from bulk expression
        (transcriptomics/proteomics) data, or spatial transcriptomics
Version: 1.2.0
Authors@R: c(
    person("Jacques","Colinge",email="jacques.colinge@inserm.fr",role="aut",comment = c(ORCID = "0000-0003-2466-4824")),
    person("Jean-Philippe","Villemin",email="jpvillemin@gmail.com",role="cre",comment = c(ORCID = "0000-0002-1838-5880")))
Description: Inference of ligand-receptor (LR) interactions from bulk
    expression (transcriptomics/proteomics) data, or spatial transcriptomics.
    BulkSignalR bases its inferences
    on the LRdb database included in our other package, SingleCellSignalR
    available from Bioconductor. It relies on a statistical model that
    is specific to bulk data sets. Different visualization and data
    summary functions are proposed to help navigating prediction results.
URL: https://github.com/ZheFrench/BulkSignalR
BugReports: https://github.com/ZheFrench/BulkSignalR/issues
License: CeCILL | file LICENSE
Encoding: UTF-8
LazyData: FALSE
Depends: R (>= 4.5)
biocViews: Network, RNASeq, Software, Proteomics, Transcriptomics,
        NetworkInference, Spatial
Imports: BiocFileCache, httr2, RCurl, cli, curl, rlang, jsonlite,
        matrixStats, methods, doParallel, glmnet, ggalluvial, ggplot2,
        gridExtra, grid, Rtsne, ggrepel, foreach, multtest, igraph,
        orthogene, stabledist, circlize (>= 0.4.14), ComplexHeatmap (>=
        2.0.0), stats, scales, RANN, SpatialExperiment,
        SummarizedExperiment, tools
Suggests: knitr, markdown, rmarkdown, STexampleData, testthat (>=
        3.0.0), codetools, Matrix, lattice, cluster, survival, MASS,
        nlme
Config/testthat/edition: 3
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/BulkSignalR
git_branch: RELEASE_3_22
git_last_commit: 4e95251
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 02:57:05 UTC; biocbuild
Author: Jacques Colinge [aut] (ORCID: <https://orcid.org/0000-0003-2466-4824>),
  Jean-Philippe Villemin [cre] (ORCID:
    <https://orcid.org/0000-0002-1838-5880>)
Maintainer: Jean-Philippe Villemin <jpvillemin@gmail.com>
