# Generated by roxygen2: do not edit by hand

export(BSRClusterComp)
export(BSRDataModel)
export(BSRDataModelComp)
export(BSRInference)
export(BSRInferenceComp)
export(BSRSignature)
export(BSRSignatureComp)
export(alluvialPlot)
export(assignCellTypesToInteractions)
export(bubblePlotPathwaysLR)
export(cacheClear)
export(cacheInfo)
export(cacheVersion)
export(cellTypeFrequency)
export(cellularNetwork)
export(cellularNetworkTable)
export(chordDiagramLR)
export(convertToHuman)
export(createResources)
export(findOrthoGenes)
export(generateSpatialPlots)
export(getLRIntracellNetwork)
export(getLRNetwork)
export(getResource)
export(maxLigandSpatialCounts)
export(relateToGeneSet)
export(resetLRdb)
export(resetNetwork)
export(resetPathways)
export(scoreSignatures)
export(separatedLRPlot)
export(signatureHeatmaps)
export(simpleHeatmap)
export(smoothSpatialCounts)
export(spatialAssociation)
export(spatialAssociationPlot)
export(spatialDiversityPlot)
export(spatialIndexPlot)
export(spatialPlot)
export(summarizedCellularNetwork)
exportClasses(BSRClusterComp)
exportClasses(BSRDataModel)
exportClasses(BSRDataModelComp)
exportClasses(BSRInference)
exportClasses(BSRInferenceComp)
exportClasses(BSRSignature)
exportClasses(BSRSignatureComp)
exportMethods(LRinter)
exportMethods(LRinterScore)
exportMethods(LRinterShort)
exportMethods(addClusterComp)
exportMethods(coerce)
exportMethods(colClusterA)
exportMethods(colClusterB)
exportMethods(comparison)
exportMethods(comparisonName)
exportMethods(differentialStats)
exportMethods(getPathwayStats)
exportMethods(inferenceParameters)
exportMethods(initialOrganism)
exportMethods(initialOrthologs)
exportMethods(learnParameters)
exportMethods(ligands)
exportMethods(logTransformed)
exportMethods(mu)
exportMethods(ncounts)
exportMethods(normalization)
exportMethods(parameters)
exportMethods(pathways)
exportMethods(receptors)
exportMethods(reduceToBestPathway)
exportMethods(reduceToLigand)
exportMethods(reduceToPathway)
exportMethods(reduceToReceptor)
exportMethods(removeClusterComp)
exportMethods(rescoreInference)
exportMethods(resetToInitialOrganism)
exportMethods(scoreLRGeneSignatures)
exportMethods(sourceComparisonName)
exportMethods(tgCorr)
exportMethods(tgExpr)
exportMethods(tgGenes)
exportMethods(tgLogFC)
exportMethods(tgPval)
exportMethods(updateInference)
import(BiocFileCache)
import(ComplexHeatmap)
import(SpatialExperiment)
import(SummarizedExperiment)
import(doParallel)
import(ggalluvial)
import(ggplot2)
import(glmnet)
import(grid)
import(httr2)
import(igraph)
import(jsonlite)
import(multtest)
importFrom(BiocFileCache,BiocFileCache)
importFrom(BiocFileCache,bfcinfo)
importFrom(BiocFileCache,bfcquery)
importFrom(BiocFileCache,bfcremove)
importFrom(BiocFileCache,removebfc)
importFrom(ComplexHeatmap,"%v%")
importFrom(RANN,nn2)
importFrom(RCurl,url.exists)
importFrom(Rtsne,Rtsne)
importFrom(circlize,chordDiagramFromDataFrame)
importFrom(circlize,circos.axis)
importFrom(circlize,circos.clear)
importFrom(circlize,circos.par)
importFrom(circlize,circos.text)
importFrom(circlize,circos.trackPlotRegion)
importFrom(circlize,colorRamp2)
importFrom(circlize,get.cell.meta.data)
importFrom(cli,cli_alert)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_inform)
importFrom(curl,has_internet)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(ggrepel,geom_text_repel)
importFrom(gridExtra,grid.arrange)
importFrom(matrixStats,colQuantiles)
importFrom(matrixStats,colSums2)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowSums2)
importFrom(methods,is)
importFrom(methods,new)
importFrom(orthogene,convert_orthologs)
importFrom(rlang,.data)
importFrom(scales,rescale)
importFrom(stabledist,dstable)
importFrom(stabledist,pstable)
importFrom(stats,quantile)
