% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BSRDataModel-class.R
\name{ncounts<-,BSRDataModel-method}
\alias{ncounts<-,BSRDataModel-method}
\title{Normalized count matrix setter (internal use only)}
\usage{
\S4method{ncounts}{BSRDataModel}(x) <- value
}
\arguments{
\item{x}{object BSRDataModel}

\item{value}{value to be set for BSRDataModel}
}
\value{
returns \code{NULL}
}
\description{
Normalized count matrix setter (internal use only)
}
\keyword{internal}
